@@ -3,7 +3,9 @@
 
 #include "qmediaplayer_p.h"
 
+#include <private/qmultimediautils_p.h>
 #include <private/qplatformmediaintegration_p.h>
+#include <private/qaudiobufferoutput_p.h>
 #include <qvideosink.h>
 #include <qaudiooutput.h>
 
@@ -11,10 +13,10 @@
 #include <QtCore/qmetaobject.h>
 #include <QtCore/qtimer.h>
 #include <QtCore/qdebug.h>
+#include <QtCore/qdir.h>
 #include <QtCore/qpointer.h>
 #include <QtCore/qfileinfo.h>
 #include <QtCore/qtemporaryfile.h>
-#include <QtCore/qdir.h>
 #include <QtCore/qcoreapplication.h>
 
 #if defined(Q_OS_ANDROID)
@@ -44,7 +46,7 @@ QT_BEGIN_NAMESPACE
 
 /*!
     \qmltype MediaPlayer
-    \instantiates QMediaPlayer
+    \nativetype QMediaPlayer
     \brief Adds media playback to a scene.
 
     \inqmlmodule QtMultimedia
@@ -97,13 +99,15 @@ QT_BEGIN_NAMESPACE
     \sa AudioOutput, VideoOutput
 */
 
-void QMediaPlayerPrivate::setState(QMediaPlayer::PlaybackState ps)
+void QMediaPlayerPrivate::setState(QMediaPlayer::PlaybackState toState)
 {
     Q_Q(QMediaPlayer);
 
-    if (ps != state) {
-        state = ps;
-        emit q->playbackStateChanged(ps);
+    if (toState != state) {
+        const auto fromState = std::exchange(state, toState);
+        if (toState == QMediaPlayer::PlayingState || fromState == QMediaPlayer::PlayingState)
+            emit q->playingChanged(toState == QMediaPlayer::PlayingState);
+        emit q->playbackStateChanged(toState);
     }
 }
 
@@ -114,18 +118,17 @@ void QMediaPlayerPrivate::setStatus(QMediaPlayer::MediaStatus s)
     emit q->mediaStatusChanged(s);
 }
 
-void QMediaPlayerPrivate::setError(int error, const QString &errorString)
+void QMediaPlayerPrivate::setError(QMediaPlayer::Error error, const QString &errorString)
 {
     Q_Q(QMediaPlayer);
 
-    this->error = QMediaPlayer::Error(error);
-    this->errorString = errorString;
-    emit q->errorChanged();
-    emit q->errorOccurred(this->error, errorString);
+    this->error.setAndNotify(error, errorString, *q);
 }
 
 void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
 {
+    setError(QMediaPlayer::NoError, {});
+
     if (!control)
         return;
 
@@ -134,7 +137,8 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
     // Back ends can't play qrc files directly.
     // If the back end supports StreamPlayback, we pass a QFile for that resource.
     // If it doesn't, we copy the data to a temporary file and pass its path.
-    if (!media.isEmpty() && !stream && media.scheme() == QLatin1String("qrc")) {
+    if (!media.isEmpty() && !stream && media.scheme() == QLatin1String("qrc")
+        && !control->canPlayQrc()) {
         qrcMedia = media;
 
         file.reset(new QFile(QLatin1Char(':') + media.path()));
@@ -187,25 +191,14 @@ void QMediaPlayerPrivate::setMedia(const QUrl &media, QIODevice *stream)
             qWarning("Qt was built with -no-feature-temporaryfile: playback from resource file is not supported!");
 #endif
         }
-#if defined(Q_OS_ANDROID)
-    } else if (media.scheme() == QLatin1String("content") && !stream) {
-        // content scheme should happen only on android
-        const int fd = QJniObject::callStaticMethod<jint>(
-                "org/qtproject/qt/android/QtNative", "openFdForContentUrl",
-                "(Landroid/content/Context;Ljava/lang/String;Ljava/lang/String;)I",
-                QNativeInterface::QAndroidApplication::context(),
-                QJniObject::fromString(media.toString()).object(),
-                QJniObject::fromString(QLatin1String("r")).object());
-
-        file.reset(new QFile(QLatin1Char(':') + media.path()));
-        file->open(fd, QFile::ReadOnly, QFile::FileHandleFlag::AutoCloseHandle);
-        control->setMedia(media, file.get());
-#endif
     } else {
         qrcMedia = QUrl();
-        QUrl url = media;
-        if (url.scheme().isEmpty() || url.scheme() == QLatin1String("file"))
-            url = QUrl::fromUserInput(media.path(), QDir::currentPath(), QUrl::AssumeLocalFile);
+        QUrl url = qMediaFromUserInput(media);
+        if (url.scheme() == QLatin1String("content") && !stream) {
+            file.reset(new QFile(media.url()));
+            stream = file.get();
+        }
+
         control->setMedia(url, stream);
     }
 
@@ -252,9 +245,16 @@ QMediaPlayer::~QMediaPlayer()
 {
     Q_D(QMediaPlayer);
 
-    // Disconnect everything to prevent notifying
-    // when a receiver is already destroyed.
-    disconnect();
+    // prevents emitting audioOutputChanged and videoOutputChanged.
+    QSignalBlocker blocker(this);
+
+    // Reset audio output and video sink to ensure proper unregistering of the source
+    // To be investigated: registering of the source might be removed after switching on the ffmpeg
+    // backend;
+
+    // Workaround to prevent freeze in GStreamer when setting audioOutput while stopped
+    if (d->control)
+        d->control->qmediaplayerDestructorCalled = true;
     setAudioOutput(nullptr);
 
     d->setVideoSink(nullptr);
@@ -287,6 +287,8 @@ const QIODevice *QMediaPlayer::sourceDevice() const
     \property QMediaPlayer::playbackState
 
     Returns the \l{QMediaPlayer::}{PlaybackState}.
+
+    \sa playing
 */
 QMediaPlayer::PlaybackState QMediaPlayer::playbackState() const
 {
@@ -409,6 +411,12 @@ bool QMediaPlayer::isSeekable() const
     return d->control && d->control->isSeekable();
 }
 
+bool QMediaPlayer::isPlaying() const
+{
+    Q_D(const QMediaPlayer);
+    return d->state == QMediaPlayer::PlayingState;
+}
+
 /*!
     Returns the current playback rate.
 */
@@ -464,7 +472,7 @@ void QMediaPlayer::setLoops(int loops)
 */
 QMediaPlayer::Error QMediaPlayer::error() const
 {
-    return d_func()->error;
+    return d_func()->error.code();
 }
 
 /*!
@@ -481,7 +489,7 @@ QMediaPlayer::Error QMediaPlayer::error() const
 */
 QString QMediaPlayer::errorString() const
 {
-    return d_func()->errorString;
+    return d_func()->error.description();
 }
 
 /*!
@@ -489,7 +497,8 @@ QString QMediaPlayer::errorString() const
 
     Starts or resumes playback of the media.
 
-    Sets the \l playbackState property to PlayingState.
+    Sets the \l playbackState property to PlayingState, and changes
+    \l playing to \c true.
 */
 
 /*!
@@ -504,10 +513,6 @@ void QMediaPlayer::play()
     if (!d->control)
         return;
 
-    // Reset error conditions
-    d->error = NoError;
-    d->errorString = QString();
-
     d->control->play();
 }
 
@@ -516,7 +521,8 @@ void QMediaPlayer::play()
 
     Pauses playback of the media.
 
-    Sets the \l playbackState property to PausedState.
+    Sets the \l playbackState property to PausedState,
+    and changes \l playing to \c false.
 */
 
 /*!
@@ -537,7 +543,8 @@ void QMediaPlayer::pause()
 
     Stops playback of the media.
 
-    Sets the \l playbackState property to StoppedState.
+    Sets the \l playbackState property to StoppedState,
+    and changes \l playing to \c false.
 */
 
 /*!
@@ -587,12 +594,18 @@ void QMediaPlayer::setPlaybackRate(qreal rate)
 
     Setting the media to a null QUrl will cause the player to discard all
     information relating to the current media source and to cease all I/O operations related
-    to that media.
+    to that media. Setting the media will stop the playback.
 
     \note This function returns immediately after recording the specified source of the media.
     It does not wait for the media to finish loading and does not check for errors. Listen for
     the mediaStatusChanged() and error() signals to be notified when the media is loaded and
     when an error occurs during loading.
+
+    \note FFmpeg, used by the FFmpeg media backend, restricts use of nested protocols for
+    security reasons. In controlled environments where all inputs are trusted, the list of
+    approved protocols can be overridden using the QT_FFMPEG_PROTOCOL_WHITELIST environment
+    variable. This environment variable is Qt's private API and can change between patch
+    releases without notice.
 */
 
 void QMediaPlayer::setSource(const QUrl &source)
@@ -639,6 +652,84 @@ void QMediaPlayer::setSourceDevice(QIODevice *device, const QUrl &sourceUrl)
     emit sourceChanged(d->source);
 }
 
+/*!
+    \qmlproperty QAudioBufferOutput QtMultimedia::MediaPlayer::audioBufferOutput
+    \since 6.8
+
+    This property holds the target audio buffer output.
+
+    Normal usage of MediaPlayer from QML should not require using this property.
+
+    \sa QMediaPlayer::audioBufferOutput()
+*/
+
+/*!
+    \property QMediaPlayer::audioBufferOutput
+    \since 6.8
+    \brief The output audio buffer used by the media player.
+
+    Sets an audio buffer \a output to the media player.
+
+    If \l QAudioBufferOutput is specified and the media source
+    contains an audio stream, the media player, it will emit
+    the signal \l{QAudioBufferOutput::audioBufferReceived} with
+    audio buffers containing decoded audio data. At the end of
+    the audio stream, \c QMediaPlayer emits an empty \l QAudioBuffer.
+
+    \c QMediaPlayer emits outputs audio buffers at the same time as it
+    pushes the matching data to the audio output if it's specified.
+    However, the sound can be played with a small delay due to
+    audio bufferization.
+
+    The format of emitted audio buffers is taken from the
+    specified \a output or from the matching audio stream
+    if the \a output returns an invalid format. Emitted
+    audio data is not scaled depending on the current playback rate.
+
+    Potential use cases of utilizing \c QAudioBufferOutput
+    with \c QMediaPlayer might be:
+    \list
+    \li Audio visualization. If the playback rate of the media player
+    is not \c 1, you may scale the output image dimensions,
+    or image update interval according to the requirements
+    of the visualizer.
+    \li Any AI sound processing, e.g. voice recognition.
+    \li Sending the data to external audio output.
+    Playback rate changing, synchronization with video, and manual
+    flushing on stoping and seeking should be considered.
+    We don't recommend using the audio buffer output
+    for this purpose unless you have a strong reason for this.
+    \endlist
+
+*/
+void QMediaPlayer::setAudioBufferOutput(QAudioBufferOutput *output)
+{
+    Q_D(QMediaPlayer);
+
+    QAudioBufferOutput *oldOutput = d->audioBufferOutput;
+    if (oldOutput == output)
+        return;
+
+    d->audioBufferOutput = output;
+
+    if (oldOutput) {
+        auto oldPlayer = QAudioBufferOutputPrivate::exchangeMediaPlayer(*oldOutput, this);
+        if (oldPlayer)
+            oldPlayer->setAudioBufferOutput(nullptr);
+    }
+
+    if (d->control)
+        d->control->setAudioBufferOutput(output);
+
+    emit audioBufferOutputChanged();
+}
+
+QAudioBufferOutput *QMediaPlayer::audioBufferOutput() const
+{
+    Q_D(const QMediaPlayer);
+    return d->audioBufferOutput;
+}
+
 /*!
     \qmlproperty AudioOutput QtMultimedia::MediaPlayer::audioOutput
 
@@ -724,6 +815,8 @@ QList<QMediaMetaData> QMediaPlayer::audioTracks() const
 */
 
 /*!
+    \property QMediaPlayer::videoTracks
+
     Lists the set of available video tracks inside the media.
 
     The QMediaMetaData returned describes the properties of individual
@@ -988,7 +1081,7 @@ QMediaMetaData QMediaPlayer::metaData() const
 
     \value StoppedState The media player is not playing content, playback will begin from the start
     of the current track.
-    \value PlayingState The media player is currently playing content.
+    \value PlayingState The media player is currently playing content. This indicates the same as the \l playing property.
     \value PausedState The media player has paused playback, playback of the current track will
     resume from the position the player was paused at.
 */
@@ -1002,7 +1095,7 @@ QMediaMetaData QMediaPlayer::metaData() const
     \header \li Property value
             \li Description
     \row \li PlayingState
-        \li The media is currently playing.
+        \li The media is currently playing. This indicates the same as the \l playing property.
     \row \li PausedState
         \li Playback of the media has been suspended.
     \row \li StoppedState
@@ -1016,6 +1109,12 @@ QMediaMetaData QMediaPlayer::metaData() const
     This signal is emitted when the \l playbackState property is altered.
 */
 
+/*!
+    \qmlsignal QtMultimedia::MediaPlayer::playingChanged()
+
+    This signal is emitted when the \l playing property changes.
+*/
+
 /*!
     \enum QMediaPlayer::MediaStatus
 
@@ -1223,7 +1322,7 @@ QMediaMetaData QMediaPlayer::metaData() const
 
     Playback can start or resume only when the buffer is entirely filled.
     When the buffer is filled, \c MediaPlayer.Buffered is true.
-    When buffer progress is between \c 0.0 and \c 0.1, \c MediaPlayer.Buffering
+    When buffer progress is between \c 0.0 and \c 1.0, \c MediaPlayer.Buffering
     is set to \c{true}.
 
     A value lower than \c 1.0 implies that the property \c MediaPlayer.StalledMedia
@@ -1261,12 +1360,31 @@ QMediaMetaData QMediaPlayer::metaData() const
     object, use the seekableChanged signal to monitor changes.
 */
 
+/*!
+    \qmlproperty bool QtMultimedia::MediaPlayer::playing
+    \since 6.5
+
+    Indicates whether the media is currently playing.
+
+    \sa playbackState
+*/
+
+/*!
+    \property QMediaPlayer::playing
+    \brief Whether the media is playing.
+    \since 6.5
+
+    \sa playbackState, PlayingState
+*/
+
 /*!
     \qmlproperty real QtMultimedia::MediaPlayer::playbackRate
 
-    This property holds the rate at which audio is played at as a multiple of
+    This property holds the rate at which media is played at as a multiple of
     the normal rate.
 
+    For more information, see \l{QMediaPlayer::playbackRate}.
+
     Defaults to \c{1.0}.
 */
 
@@ -1274,11 +1392,11 @@ QMediaMetaData QMediaPlayer::metaData() const
     \property QMediaPlayer::playbackRate
     \brief the playback rate of the current media.
 
-    This value is a multiplier applied to the media's standard play rate. By
-    default this value is 1.0, indicating that the media is playing at the
-    standard pace. Values higher than 1.0 will increase the rate of play.
-    Values less than zero can be set and indicate the media should rewind at the
-    multiplier of the standard pace.
+    This value is a multiplier applied to the media's standard playback
+    rate. By default this value is 1.0, indicating that the media is
+    playing at the standard speed. Values higher than 1.0 will increase
+    the playback speed, while values between 0.0 and 1.0 results in
+    slower playback. Negative playback rates are not supported.
 
     Not all playback services support change of the playback rate. It is
     framework defined as to the status and quality of audio and video
